<?php

namespace Ardent\Undercurrent\Kernel;

use Ardent\Undercurrent\Container\ContainerInterface;
use Ardent\Undercurrent\Container\GenericContainer;
use Ardent\Undercurrent\Http\GenericRequest;
use Ardent\Undercurrent\Http\GenericRouter;
use Ardent\Undercurrent\Http\MethodEnum;
use Ardent\Undercurrent\Http\RouterConfig;
use Ardent\Undercurrent\Http\RouterInterface;

class BaseKernel
{
    public function __invoke(): void
    {
        $container = (new GenericContainer());
        $container
            ->alias(RouterInterface::class, GenericRouter::class)
            ->alias(ContainerInterface::class, GenericContainer::class)
            ->add(GenericContainer::class, fn($container) => $container)
            ->add(GenericRouter::class);

        $this->dependencies($container);

        $this->render($container);
    }

    private function render(GenericContainer $container): void
    {
        $request = new GenericRequest(
            MethodEnum::from($_SERVER['REQUEST_METHOD']),
            $_SERVER['REQUEST_URI'],
            $_REQUEST,
        );
        $router = $container->get(RouterInterface::class);
        $response = $router->dispatch($request);

        http_response_code($response->getStatus()->value);

        foreach ($response->getHeaders() as $header) {
            header($header);
        }

        echo $response->getBody();
    }

    protected function addControllers(ContainerInterface $container, array $controllers): void
    {
        $config = new RouterConfig();

        foreach ($controllers as $controller) {
            $container->add($controller);
            $config->addController($controller);
        }

        $container->add(RouterConfig::class, fn() => $config);
    }

    protected function dependencies(ContainerInterface $container): void
    {
    }
}